import QtQuick 2.4
import QtQml.Models 2.1
import Ubuntu.Components 1.3
import UbuntuSiloInstaller 1.0
import U1db 1.0 as U1db
import "Common.js" as Common

Page {
    id: allSilosPage
    title: i18n.tr("All Silos")
    clip: true

    head {
        sections {
            model: [i18n.tr("Installable Only"), i18n.tr("All")]
        }
    }
    property bool installableOnly: head.sections.selectedIndex == 0

    DelegateModel {
        id: silosDelegateModel

        property var silosModel: JSONListModel {
            source: "https://requests.ci-train.ubuntu.com/v1/tickets"
            // select requests that have source packages built if any
            // or at least download links that we assume to be click packages
            query: "$.requests[?(@.download_links.indexOf('.click') != -1 && @.sources == '' \
                    || @.sources != ''%1)]".arg(allSilosPage.installableOnly ? " && @.status.indexOf('Successfully built') == 0" : "")

            function itemTransform(item) {
                item["formattedDate"] = Qt.formatDate(item["date"], Qt.DefaultLocaleLongDate);
                return item;
            }

            Connections {
                target: Qt.application
                onActiveChanged: if (Qt.application.active) silosDelegateModel.silosModel.update()
            }
        }

        model: silosDelegateModel.silosModel.model
        delegate: ListItem {
            height: listItemLayout.height + divider.height

            U1db.Document {
                id: requestDocument
                database: main.localSilosDB
                docId: request_id
            }

            ListItemLayout {
                id: listItemLayout
                title.text: sources.split(' ').join(', ')
                subtitle.text: "#%1 / %2 %3".arg(request_id).arg(Common.siloNumberFromName(siloname)).arg(description)

                property bool installed: requestDocument.contents != undefined && requestDocument.contents.installed != undefined

                Icon {
                    name: "document-save"
                    SlotsLayout.position: SlotsLayout.Leading
                    height: units.gu(3)
                    visible: listItemLayout.installed
                    anchors.verticalCenter: parent.verticalCenter
                    SlotsLayout.overrideVerticalPositioning: true
                }
                ProgressionSlot {
                    anchors.verticalCenter: parent.verticalCenter
                    SlotsLayout.overrideVerticalPositioning: true
                }
            }

            onClicked: adaptivePageLayout.addPageToNextColumn(allSilosPage, Qt.resolvedUrl("SiloPage.qml"),
                                                              {"requestId": request_id})
        }
    }

    UbuntuListView {
        id: projectsListView
        anchors.fill: parent

        pullToRefresh {
            enabled: true
            refreshing: silosDelegateModel.silosModel.updating
            onRefresh: silosDelegateModel.silosModel.update()
        }

        Component.onCompleted: {
            // FIXME: workaround for qtubuntu not returning values depending on the grid unit definition
            // for Flickable.maximumFlickVelocity and Flickable.flickDeceleration
            var scaleFactor = units.gridUnit / 8;
            maximumFlickVelocity = maximumFlickVelocity * scaleFactor;
            flickDeceleration = flickDeceleration * scaleFactor;
        }


        model: silosDelegateModel
        currentIndex: -1

        Component {
            id: sectionHeading
            Rectangle {
                width: parent.width
                height: sectionLabel.height + units.gu(1)
                color: Qt.rgba(0.3, 0.3, 0.3, 0.05)

                Label {
                    id: sectionLabel
                    anchors {
                        left: parent.left
                        right: parent.right
                        margins: units.gu(2)
                        verticalCenter: parent.verticalCenter
                    }

                    text: section
                    fontSize: "x-small"
                }
            }
        }

        section.property: "formattedDate"
        section.criteria: ViewSection.FullString
        section.delegate: sectionHeading
    }

    Scrollbar {
        flickableItem: projectsListView
    }
}
