import QtQuick 2.4
import UbuntuSiloInstaller 1.0

Item {
    property url clickUrl
    property string installCommand: "UbuntuSiloInstaller/installclick %1"
    property string uninstallCommand: "UbuntuSiloInstaller/uninstallclick %1"
    property bool installing: false
    property bool uninstalling: false
    property alias output: process.output
    property alias hasError: process.hasError

    ExternalProcess {
        id: process
        onFinished: {
            installing = false;
            uninstalling = false;
        }
        onError: {
            installing = false;
            uninstalling = false;
        }
    }

    function install () {
        if (installing) {
            console.warn("Cannot install: installation already in progress.");
            return;
        }
        if (uninstalling) {
            console.warn("Cannot install: uninstallation in progress.");
            return;
        }
        if (clickUrl == "") {
            console.warn("Cannot install: no clickUrl provided.");
            return;
        }

        process.execute(installCommand.arg(clickUrl));
        installing = true;
    }

    function uninstall () {
        if (installing) {
            console.warn("Cannot uninstall: installation in progress.");
            return;
        }
        if (uninstalling) {
            console.warn("Cannot uninstall: uninstallation already in progress.");
            return;
        }
        if (clickUrl == "") {
            console.warn("Cannot uninstall: no clickUrl provided.");
            return;
        }

        process.execute(uninstallCommand.arg(clickUrl));
        uninstalling = true;
    }
}

