import QtQuick 2.4
import QtQuick.Layouts 1.1
import QtQml.Models 2.1
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3
import UbuntuSiloInstaller 1.0
import U1db 1.0 as U1db

Page {
    id: indexPage
    title: i18n.tr("Ubuntu Silo Installer")
    clip: true

    DelegateModel {
        id: installedSilosCounter
        model: U1db.Query{
            index: U1db.Index{
                database: main.localSilosDB
                expression: ["request_id"]
            }
        }
        delegate: QtObject {}
    }

    UbuntuListView {
        id: indexListView
        anchors {
            left: parent.left
            right: parent.right
        }
        height: units.gu(25)
        enabled: !flasher.flashing

        model: ListModel {
            ListElement {
                title: QT_TR_NOOP("Projects")
                page: "ProjectsPage.qml"
            }
            ListElement {
                title: QT_TR_NOOP("All Silos")
                page: "AllSilosPage.qml"
            }
            ListElement {
                title: QT_TR_NOOP("Installed Silos")
                page: "InstalledSilos.qml"
            }
        }
        currentIndex: -1

        delegate: ListItem {
            height: listItemLayout.height + divider.height
            enabled: model.page != "InstalledSilos.qml" || installedSilosCounter.count != 0

            ListItemLayout {
                id: listItemLayout
                title.text: i18n.tr(model.title)
                opacity: enabled ? 1.0 : 0.5

                Label {
                    visible: model.page == "InstalledSilos.qml"
                    text: installedSilosCounter.count
                }

                ProgressionSlot {}
            }

            onClicked: adaptivePageLayout.addPageToCurrentColumn(indexPage,
                                                                 Qt.resolvedUrl(model.page))
        }
    }

    ColumnLayout {
        anchors {
            top: indexListView.bottom
            topMargin: units.gu(4)
            left: parent.left
            right: parent.right
            bottom: parent.bottom
            bottomMargin: units.gu(2)
        }
        spacing: units.gu(2)

        Button {
            anchors {
                horizontalCenter: parent.horizontalCenter
            }
            visible: installedSilosCounter.count != 0
            text: i18n.tr("Forget Installed Silos")
            onTriggered: main.localSilosDB.empty()
        }

        Button {
            anchors {
                horizontalCenter: parent.horizontalCenter
            }
            text: flasher.flashing ? i18n.tr("Reflashing takes several minutes") : i18n.tr("Reflash device")
            color: UbuntuColors.red
            enabled: !flasher.flashing
            onTriggered: PopupUtils.open(flashConfirmationDialogComponent)
        }

        Component {
             id: flashConfirmationDialogComponent
             Dialog {
                 id: flashConfirmationDialog
                 title: i18n.tr("Are you sure?")
                 text: i18n.tr("This will erase and reinstall the system. " +
                                "However, your data and your installed applications " +
                                "will be preserved.")
                 Button {
                     text: i18n.tr("Cancel")
                     onClicked: PopupUtils.close(flashConfirmationDialog)
                 }
                 Button {
                     text: i18n.tr("Reflash")
                     color: UbuntuColors.red
                     onClicked: {
                         flasher.cleanAndFlash();
                         PopupUtils.close(flashConfirmationDialog);
                     }
                 }
             }
        }

        Terminal {
            anchors {
                left: parent.left
                right: parent.right
                leftMargin: units.gu(2)
                rightMargin: units.gu(2)
            }
            Layout.fillHeight: true
            opacity: flasher.hasError || flasher.flashing ? 1.0 : 0.0
            // FIXME: thoroughly inefficient
            text: flasher.output.join("")
            hasError: flasher.hasError

            ExternalProcess {
                id: flasher
                property string sudoCommand: "UbuntuSiloInstaller/customsudo %1 ".arg(main.password)
                property string flashCommand: "system-image-cli --verbose --build 0"
//                property string flashCommand: "ls"
                property bool flashing: false
                function flash() {
                    flashing = true;
                    flasher.execute(sudoCommand + flashCommand);
                }

                function uninstallClicks() {
                    // revert installation of click packages
                    var docIds = main.localSilosDB.listDocs();
                    for (var index in docIds) {
                        var docId = docIds[index];
                        var doc = main.localSilosDB.getDoc(docId);
                        if (doc.has_click) {
                            var installer = main.getInstallerForRequest(doc.request_id);
                            installer.uninstallClick();
                        }
                    }
                }

                function cleanAndFlash() {
                    uninstallClicks();
                    // FIXME: should wait until all click uninstallations are complete
                    main.localSilosDB.empty();
                    flash();
                }
            }
        }
    }
}
