import QtQuick 2.4
import QtQml.Models 2.1
import Ubuntu.Components 1.3
import UbuntuSiloInstaller 1.0
import U1db 1.0 as U1db
import "Common.js" as Common

Page {
    id: installedSilosPage
    title: i18n.tr("Installed Silos")
    clip: true

    property string projectName

    DelegateModel {
        id: installedSilosDelegateModel

        // FIXME: using U1db to filter out uninstalled silos does not work
        // Ref.: https://bugs.launchpad.net/u1db-qt/+bug/1271973
        property var by_installed: U1db.Index{
            database: main.localSilosDB
            expression: ["request_id",
                         "description",
                         "sources",
                         "installed",
                         "siloname"]
        }

        model: U1db.Query{
            index: installedSilosDelegateModel.by_installed
        }

        delegate: ListItem {
            height: listItemLayout.height + divider.height

            ListItemLayout {
                id: listItemLayout
                title.text: contents.sources.split(' ').join(', ')
                subtitle.text: "#%1 / %2 %3".arg(contents.request_id).arg(Common.siloNumberFromName(contents.siloname)).arg(contents.description)
                ProgressionSlot {
                    anchors.verticalCenter: parent.verticalCenter
                    SlotsLayout.overrideVerticalPositioning: true
                }
            }

            onClicked: adaptivePageLayout.addPageToNextColumn(installedSilosPage, Qt.resolvedUrl("SiloPage.qml"),
                                                              {"requestId": contents.request_id})
        }
    }

    UbuntuListView {
        id: projectsListView
        anchors.fill: parent

        Component.onCompleted: {
            // FIXME: workaround for qtubuntu not returning values depending on the grid unit definition
            // for Flickable.maximumFlickVelocity and Flickable.flickDeceleration
            var scaleFactor = units.gridUnit / 8;
            maximumFlickVelocity = maximumFlickVelocity * scaleFactor;
            flickDeceleration = flickDeceleration * scaleFactor;
        }


        currentIndex: -1
        model: installedSilosDelegateModel
    }

    Scrollbar {
        flickableItem: projectsListView
    }
}
