import QtQuick 2.4
import Ubuntu.Components 1.3
import UbuntuSiloInstaller 1.0
import U1db 1.0 as U1db
import "Common.js" as Common

Page {
    id: projectSilosPage
    title: i18n.tr("Silos for %1").arg(projectName)
    clip: true

    property string projectName

    Connections {
        target: Qt.application
        onActiveChanged: if (Qt.application.active) silosModel.update()
    }

    UbuntuListView {
        id: projectsListView
        anchors.fill: parent

        pullToRefresh {
            enabled: true
            refreshing: silosModel.updating
            onRefresh: silosModel.update()
        }

        Component.onCompleted: {
            // FIXME: workaround for qtubuntu not returning values depending on the grid unit definition
            // for Flickable.maximumFlickVelocity and Flickable.flickDeceleration
            var scaleFactor = units.gridUnit / 8;
            maximumFlickVelocity = maximumFlickVelocity * scaleFactor;
            flickDeceleration = flickDeceleration * scaleFactor;
        }

        JSONListModel {
            id: silosModel
            source: "https://requests.ci-train.ubuntu.com/v1/tickets"
            query: "$.requests[?(@.sources.indexOf('%1') !=-1)]".arg(projectSilosPage.projectName)
        }
        model: silosModel.model
        currentIndex: -1

        delegate: ListItem {
            height: listItemLayout.height + divider.height

            U1db.Document {
                id: requestDocument
                database: main.localSilosDB
                docId: request_id
            }

            ListItemLayout {
                id: listItemLayout
//                title.text: sources.split(' ').join(', ')
                title.text: "#%1 / %2 %3".arg(request_id).arg(Common.siloNumberFromName(siloname)).arg(description)

                property bool installed: requestDocument.contents != undefined && requestDocument.contents.installed != undefined

                Icon {
                    name: "document-save"
                    SlotsLayout.position: SlotsLayout.Leading
                    height: units.gu(3)
                    visible: listItemLayout.installed
                }

                ProgressionSlot {}
            }

            onClicked: adaptivePageLayout.addPageToNextColumn(projectSilosPage, Qt.resolvedUrl("SiloPage.qml"),
                                                              {"requestId": request_id})
        }
    }

    Scrollbar {
        flickableItem: projectsListView
    }
}
