import QtQuick 2.4
import QtQml.Models 2.1
import Ubuntu.Components 1.3
import UbuntuSiloInstaller 1.0

Page {
    id: projectsPage
    title: i18n.tr("Projects")
    clip: true

    head {
        sections {
            model: [i18n.tr("Installable Only"), i18n.tr("All")]
        }
    }
    property bool installableOnly: head.sections.selectedIndex == 0

    DelegateModel {
        id: projectsDelegateModel

        property var projectsModel: JSONListModel {
            source: "https://requests.ci-train.ubuntu.com/v1/tickets"
            // select requests that have source packages built if any
            // or at least download links that we assume to be click packages
            query: "$.requests[?(@.download_links.indexOf('.click') != -1 && @.sources == '' \
                    || @.sources != ''%1)].sources".arg(projectsPage.installableOnly ? " && @.status.indexOf('Successfully built') == 0" : "")

            function firstWord(str) {
                return str.split(' ')[0];
            }

            function uniqueValues(array) {
                return array.reverse().filter(function (e, i, array) {
                    return array.indexOf(e, i+1) === -1 && e;
                });
            }

            function flattenProjects(projects) {
                return projects.reduce(function (flat, toFlatten) {
                    if (!Array.isArray(flat)) {
                        flat = flat.split(' ');
                    }
                    return flat.concat(toFlatten.split(' '));
                });
            }

            property var projects: projectsDelegateModel.projectsModel.rawObjectArray ?
                                       uniqueValues(flattenProjects(projectsDelegateModel.projectsModel.rawObjectArray)).sort() : null

            Connections {
                target: Qt.application
                onActiveChanged: if (Qt.application.active) projectsDelegateModel.projectsModel.update()
            }
        }

        model: projectsDelegateModel.projectsModel.rawObjectArray ? projectsDelegateModel.projectsModel.projects : null
        delegate: ListItem {
            height: listItemLayout.height + divider.height

            property string projectName: modelData
            ListItemLayout {
                id: listItemLayout
                title.text: projectName
                subtitle.text: modelData != projectName ? modelData.split(' ').slice(1).join(', ') : ""
                ProgressionSlot {}
            }

            onClicked: adaptivePageLayout.addPageToCurrentColumn(projectsPage, Qt.resolvedUrl("ProjectSilosPage.qml"),
                                                              {"projectName": projectName})
        }
    }

    UbuntuListView {
        id: projectsListView
        anchors.fill: parent

        pullToRefresh {
            enabled: true
            refreshing: projectsDelegateModel.projectsModel.updating
            onRefresh: projectsDelegateModel.projectsModel.update()
        }

        Component.onCompleted: {
            // FIXME: workaround for qtubuntu not returning values depending on the grid unit definition
            // for Flickable.maximumFlickVelocity and Flickable.flickDeceleration
            var scaleFactor = units.gridUnit / 8;
            maximumFlickVelocity = maximumFlickVelocity * scaleFactor;
            flickDeceleration = flickDeceleration * scaleFactor;
        }

        currentIndex: projectsDelegateModel.projectsModel.rawObjectArray ? -1 : 0
        model: projectsDelegateModel
    }

    Scrollbar {
        flickableItem: projectsListView
    }
}
