import QtQuick 2.4
import Ubuntu.Components 1.3
import UbuntuSiloInstaller 1.0
import U1db 1.0 as U1db
import "Common.js" as Common

QtObject {
    id: requestInstaller

    property string requestId
    property var request: siloModel.model.count != 0 ? siloModel.model.get(0) : null
    property bool installed: requestDoc != undefined && requestDoc.installed != undefined
    property bool installing: siloInstaller.installing || clickInstaller.installing
    property bool uninstalling: siloInstaller.uninstalling || clickInstaller.uninstalling
    property bool hasError: siloInstaller.hasError || clickInstaller.hasError
    property bool hasDownload: request && request.download_links.indexOf('.click') != -1
    property bool hasSourcePackages: request && request.sources != ""
    property string output
    property var requestDoc
    onRequestChanged: if (request) requestDoc = main.localSilosDB.getDoc(request.request_id)

    property var siloInstaller: SiloInstaller {
        siloNumber: request ? Common.siloNumberFromName(request.siloname) : -1
        onFinishedInstalling: installClick()
        onFinishedUninstalling: uninstallClick()
    }

    property var clickInstaller: ClickInstaller {
        clickUrl: request ? request.download_links : ""
    }

    property var siloModel: JSONListModel {
        source: requestId ? "https://requests.ci-train.ubuntu.com/v1/ticket/%1".arg(requestId) : ""
        //query: "$.requests[?(@.sources.indexOf('%1') !=-1)]".arg(project)
        query: "$.requests"

        Connections {
            target: Qt.application
            onActiveChanged: if (Qt.application.active) siloModel.update()
        }
    }

    onInstallingChanged: {
        if (!installing && !hasError) {
            var doc = {};
            doc["request_id"] = request.request_id;
            doc["description"] = request.description;
            doc["sources"] = request.sources;
            doc["installed"] = true;
            doc["has_click"] = requestInstaller.hasDownload;
            doc["siloname"] = request.siloname;
            main.localSilosDB.putDoc(doc, request.request_id);
            requestDoc = doc;
        }
    }

    onUninstallingChanged: {
        if (!uninstalling && !hasError) {
            main.localSilosDB.deleteDoc(request.request_id);
            requestDoc = undefined;
        }
    }

    function installSilo() {
        if (hasSourcePackages) {
            // Should install relevant debs from PPA using citrain
            siloInstaller.install();
            requestInstaller.output = Qt.binding(function () {return siloInstaller.output.join("")});
        }
    }

    function installClick() {
        if (hasDownload) {
            var url = request.download_links;
            // Should download url, assume the file is a click and install it
            clickInstaller.install();
            requestInstaller.output = Qt.binding(function () {return clickInstaller.output.join("")});
        }
    }

    function uninstallSilo() {
        if (hasSourcePackages) {
            // Should uninstall debs coming from PPA using citrain
            siloInstaller.uninstall();
            requestInstaller.output = Qt.binding(function () {return siloInstaller.output.join("")});
        }
    }

    function uninstallClick() {
        if (hasDownload) {
            clickInstaller.uninstall();
            requestInstaller.output = Qt.binding(function () {return clickInstaller.output.join("")});
        }
    }

    function install() {
        if (hasSourcePackages) {
            installSilo();
        } else {
            installClick();
        }
    }

    function uninstall() {
        if (hasSourcePackages) {
            uninstallSilo();
        } else {
            uninstallClick();
        }
    }
}
