import QtQuick 2.4
import UbuntuSiloInstaller 1.0

Item {
    id: siloInstaller

    property int siloNumber: -1
    property string installCommand: "UbuntuSiloInstaller/customcitrain host-upgrade %1 %2"
//    property string installCommand: "UbuntuSiloInstaller/customcitrain"
//    property string installCommand: "cat /var/log/syslog"
//    property string installCommand: "ls"
    property string uninstallCommand: "UbuntuSiloInstaller/customcitrain host-purge %1 %2"
//    property string uninstallCommand: "ls"
    property bool installing: false
    property bool uninstalling: false
    property alias output: process.output
    property alias hasError: process.hasError
    signal finishedInstalling
    signal errorInstalling
    signal finishedUninstalling
    signal errorUninstalling

    ExternalProcess {
        id: process
        onFinished: {
            if (installing) {
                siloInstaller.finishedInstalling();
            }
            if (uninstalling) {
                siloInstaller.finishedUninstalling();
            }

            installing = false;
            uninstalling = false;
        }
        onError: {
            if (installing) {
                siloInstaller.errorInstalling();
            }
            if (uninstalling) {
                siloInstaller.errorUninstalling();
            }

            installing = false;
            uninstalling = false;
        }
    }

    function install () {
        if (installing) {
            console.warn("Cannot install: installation already in progress.");
            return;
        }
        if (uninstalling) {
            console.warn("Cannot install: uninstallation in progress.");
            return;
        }
        if (siloNumber == -1) {
            console.warn("Cannot install: no silo number provided.");
            return;
        }

        process.execute(installCommand.arg(siloNumber).arg(main.password));
        installing = true;
    }

    function uninstall () {
        if (installing) {
            console.warn("Cannot uninstall: installation in progress.");
            return;
        }
        if (uninstalling) {
            console.warn("Cannot uninstall: uninstallation already in progress.");
            return;
        }
        if (siloNumber == -1) {
            console.warn("Cannot uninstall: no silo number provided.");
            return;
        }
        process.execute(uninstallCommand.arg(siloNumber).arg(main.password));
        uninstalling = true;
    }
}

