import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Themes.Ambiance 1.3
import UbuntuSiloInstaller 1.0
import "Common.js" as Common

Page {
    id: siloPage
    clip: true
    title: i18n.tr("Request #%1 / Silo %2").arg(requestId).arg(Common.siloNumberFromName(request.siloname))

    property string requestId
    property var request: installer ? installer.request : null
    property bool hasDownload: request && request.download_links.indexOf(".click") != -1
    property bool hasSourcePackages: request && request.sources != ""
    property bool canBeInstalled: hasSourcePackages ? request.status == "Successfully built" : hasDownload
    property var installer: main.getInstallerForRequest(requestId)

    head.actions: [
        Action {
            iconName: "external-link"
            text: i18n.tr("Go To Train")
            onTriggered: Qt.openUrlExternally("https://requests.ci-train.ubuntu.com/#/ticket/%1".arg(siloPage.request.request_id))
        }
    ]

    Flickable {
        id: siloFlickable
        anchors.fill: parent
        contentHeight: contentItem.childrenRect.height
        topMargin: units.gu(2)
        bottomMargin: units.gu(2)

        Component.onCompleted: {
            // FIXME: workaround for qtubuntu not returning values depending on the grid unit definition
            // for Flickable.maximumFlickVelocity and Flickable.flickDeceleration
            var scaleFactor = units.gridUnit / 8;
            maximumFlickVelocity = maximumFlickVelocity * scaleFactor;
            flickDeceleration = flickDeceleration * scaleFactor;
        }

        Column {
            anchors {
                left: parent.left
                right: parent.right
                margins: units.gu(2)
            }
            spacing: units.gu(2)
            onHeightChanged: {
                if (installerOutput.visible
                    && siloFlickable.contentY + siloFlickable.height < siloFlickable.contentHeight) {
                    // scroll terminal into view
                    siloFlickable.flick(0, -units.dp(1000));
                }
            }

            Label {
                anchors {
                    left: parent.left
                    right: parent.right
                }

                font.bold: true
                text: siloPage.request ? siloPage.request.sources.split(' ').join('\n') : ""
                visible: text != ""
            }

            Label {
                anchors {
                    left: parent.left
                    right: parent.right
                }

                text: siloPage.request ? i18n.tr("Submitted %1\nby %2")
                                           .arg(Qt.formatDate(siloPage.request.date, Qt.DefaultLocaleLongDate))
                                           .arg(siloPage.request.creator) : ""
            }

            Label {
                anchors {
                    left: parent.left
                    right: parent.right
                }

                text: siloPage.request ? siloPage.request.status.split('. ').join('.\n') : ""
                color: siloPage.request ? siloPage.request.status_color : ""
                visible: siloPage.hasSourcePackages && text != ""
                wrapMode: Text.Wrap
            }

            Label {
                anchors {
                    left: parent.left
                    right: parent.right
                }

                text: siloPage.request ? siloPage.request.description : ""
                wrapMode: Text.Wrap
            }

            Row {
                spacing: units.gu(2)
                anchors.horizontalCenter: parent.horizontalCenter

                Button {
                    id: installButton
                    text: i18n.tr("Install")
                    enabled: !installer.installing
                    color: siloPage.canBeInstalled ? UbuntuColors.green :
                                                       UbuntuColors.red
                    visible: siloPage.request && !installer.installed
                    onClicked: installer.install()
                }

                Button {
                    id: uninstallButton
                    text: i18n.tr("Uninstall")
                    enabled: !installer.uninstalling
                    visible: siloPage.request && installer.installed
                    color: UbuntuColors.red
                    onClicked: installer.uninstall()
                }

                Button {
                    id: upgradeButton
                    text: i18n.tr("Upgrade")
                    enabled: !installer.uninstalling
                    visible: siloPage.request && installer.installed
                    color: UbuntuColors.lightGrey
                    onClicked: installer.install()
                }

                ActivityIndicator {
                    anchors.verticalCenter: parent.verticalCenter
                    running: installer.installing || installer.uninstalling
                }
            }

            Terminal {
                id: installerOutput
                anchors {
                    left: parent.left
                    right: parent.right
                }
                visible: installer.hasError || installer.installing || installer.uninstalling || installer.output
                // FIXME: thoroughly inefficient
                text: installer.output
                hasError: installer.hasError
            }

        }
    }

    Scrollbar {
        flickableItem: siloFlickable
    }
}
