#include "external_process.h"
#include <QDebug>
#include <QUrl>
#include <QtQml>

ExternalProcess::ExternalProcess(QObject *parent) :
    QObject(parent),
    m_hasError(false),
    m_executing(false)
{
    m_process.setProcessChannelMode(QProcess::MergedChannels);
    m_process.setReadChannel(QProcess::StandardOutput);
    QObject::connect(&m_process, &QProcess::readyRead,
                     this, &ExternalProcess::readAvailableLinesToOutput);
    QObject::connect(&m_process, static_cast<void(QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished),
                     this, &ExternalProcess::onProcessFinished);
    QObject::connect(&m_process, static_cast<void(QProcess::*)(QProcess::ProcessError)>(&QProcess::error),
                     this, &ExternalProcess::onProcessError);
}

ExternalProcess::~ExternalProcess()
{

}

bool ExternalProcess::hasError() const
{
    return m_hasError;
}

bool ExternalProcess::executing() const
{
    return m_executing;
}

bool ExternalProcess::execute(QString command)
{
    setHasError(false);
    m_output.clear();
    Q_EMIT outputChanged();
    m_executing = true;
    Q_EMIT executingChanged();
    m_process.start(command);
}

void ExternalProcess::readAvailableLinesToOutput()
{
    while (m_process.canReadLine()) {
        QString line = QString::fromLocal8Bit(m_process.readLine());
        if (!line.isEmpty()) {
            fprintf(stdout, line.toLocal8Bit().constData());
            fflush(stdout);
            m_output.append(line);
        }
    }
    Q_EMIT outputChanged();
}

void ExternalProcess::onProcessError(QProcess::ProcessError error)
{
    Q_UNUSED(error)
    setHasError(true);
    m_executing = false;
    Q_EMIT executingChanged();
    Q_EMIT ExternalProcess::error();
}

void ExternalProcess::onProcessFinished(int exitCode, QProcess::ExitStatus exitStatus)
{
    Q_UNUSED(exitStatus)
    if (exitCode != 0) {
        setHasError(true);
    }
    m_executing = false;
    Q_EMIT executingChanged();
    Q_EMIT finished(exitCode);
}

void ExternalProcess::setHasError(bool hasError)
{
    if (hasError != m_hasError) {
        m_hasError = hasError;
        Q_EMIT hasErrorChanged();
    }
}

QStringList ExternalProcess::output() const
{
    return m_output;
}
