#ifndef EXTERNALPROCESS_H
#define EXTERNALPROCESS_H

#include <QObject>
#include <QProcess>

class ExternalProcess : public QObject
{
    Q_OBJECT

    Q_PROPERTY(QStringList output READ output NOTIFY outputChanged)
    Q_PROPERTY(bool hasError READ hasError NOTIFY hasErrorChanged)
    Q_PROPERTY(bool executing READ executing NOTIFY executingChanged)

public:
    explicit ExternalProcess(QObject *parent = 0);
    ~ExternalProcess();

    QStringList output() const;
    bool hasError() const;
    bool executing() const;
    Q_INVOKABLE bool execute(QString command);

Q_SIGNALS:
    void outputChanged();
    void hasErrorChanged();
    void error();
    void finished(int exitCode);
    void executingChanged();

protected:
    Q_SLOT void readAvailableLinesToOutput();
    Q_SLOT void onProcessError(QProcess::ProcessError error);
    Q_SLOT void onProcessFinished(int exitCode, QProcess::ExitStatus exitStatus);
    void setHasError(bool hasError);

protected:
    QProcess m_process;
    QStringList m_output;
    bool m_hasError;
    bool m_executing;
};

#endif // EXTERNALPROCESS_H
